package net.jaekl.cfb.db;


public class Row {
	Column[] m_columns;
	Object[] m_values;
	
	public Row(Column[] columns, Object[] values)
	{
		m_columns = columns.clone();
		m_values = values.clone();
	}
	
	public int getNumColumns() { return m_columns.length; }
	public Column getColumn(int idx) { return m_columns[idx]; }
	
	public String getString(int index) throws TypeMismatchException {
		checkType(index, Column.Type.VARCHAR);
		return (String)m_values[index];
	}
	
	public int getInt(int index) throws TypeMismatchException
	{
		checkType(index, Column.Type.INTEGER);
		Number num = (Number)m_values[index];
		return num.intValue();
	}
	
	public long getLong(int index) throws TypeMismatchException
	{
		checkType(index, Column.Type.INTEGER);
		Number num = (Number)m_values[index];
		return num.longValue();
	}
	
	public java.util.Date getDate(int index) throws TypeMismatchException
	{
		checkType(index, Column.Type.INTEGER);
		long milliseconds = (Long)m_values[index];
		java.util.Date date = new java.util.Date(milliseconds);
		return date;
	}
	
	protected void checkType(int index, Column.Type type) throws TypeMismatchException {
		Column column = m_columns[index];
		if (column.getType().equals(type)) {
			return;
		}
		
		String msg = "Column " + column.getName() 
				   + " is of type " + column.getType().name()
				   + " which cannot be coerced to type " + type.name() + ".";
		throw new TypeMismatchException(msg);
	}
}
