// Copyright (C) 2004, 2015 Christian Jaekl

package net.jaekl.qd.xml;

import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

public class ParseHandler implements ContentHandler
{
	Stack<ParseResult> m_stack;

	public ParseHandler(ParseResult root) {
		m_stack = new Stack<ParseResult>();
		m_stack.push(root);
	}

	@Override 
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		if (m_stack.isEmpty()) {
			return;
		}
		
		try {
			m_stack.peek().characters(ch, start, length);
		}
		catch (XmlParseException xpe) {
			throw new SAXException(xpe);
		}
	}

	@Override 
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		try {
			if (m_stack.isEmpty()) {
				return;
			}
			
			boolean pop = m_stack.peek().endElement(uri, localName, qName);
			if (pop) {
				m_stack.pop();

				if (m_stack.isEmpty()) {
					return;
				}
				
				m_stack.peek().endExternal(uri, localName, qName);
			}
		}
		catch (XmlParseException xpe) {
			throw new SAXException(xpe);
		}
	}

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		try {
			ParseResult current = m_stack.peek();
			ParseResult next = current.startElement(uri, localName, qName, attributes);
			if (next != current) {
				m_stack.push(next);
			}
		}
		catch (XmlParseException xpe) {
			throw new SAXException(xpe);
		}
	}

	@Override
	public void endDocument() throws SAXException {
		if (! m_stack.isEmpty()) {
			String missingTag = m_stack.peek().getTagName();
			throw new SAXException(new MissingInfoException(missingTag));
		}
	}

	@Override
	public void endPrefixMapping(String prefix) throws SAXException {
		// no-op
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length)
	throws SAXException 
	{
		// no-op
	}

	@Override
	public void processingInstruction(String target, String data)
	throws SAXException 
	{
		// no-op
	}

	@Override
	public void setDocumentLocator(Locator locator) {
		// no-op
	}

	@Override
	public void skippedEntity(String name) throws SAXException {
		// no-op
	}

	@Override
	public void startDocument() throws SAXException {
		// no-op
	}

	@Override
	public void startPrefixMapping(String prefix, String uri)
	throws SAXException 
	{
		// no-op
	}
}
