package net.jaekl.cfb.db;

import static org.junit.Assert.*;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

import net.jaekl.cfb.analyze.MessageMap;
import net.jaekl.cfb.db.driver.ConnectionMock;
import net.jaekl.cfb.db.driver.DbDriverMock;
import net.jaekl.cfb.xml.MessagesXmlData;

import org.junit.Test;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class CfbSchemaTest {
	@Test
	public void testEnsureDbInitialized() throws SQLException, FileNotFoundException, IOException, SAXException {
		ConnectionMock con;
		DbDriverMock dbDriver;
		CfbSchema schema;
		con = new ConnectionMock();
		dbDriver = new DbDriverMock();
		schema = new CfbSchema(dbDriver);
		
		MessageMap msgMap = new MessageMap();
		msgMap.parse(new InputSource(new ByteArrayInputStream(MessagesXmlData.XML.getBytes("UTF-8"))));
		
		schema.setMessageMap(msgMap);
		schema.ensureDbInitialized(con);
		
		for (Table table : CfbSchema.TABLES) {
			assertTablePresent(con, dbDriver, table);
		}
	}
	
	private void assertTablePresent(ConnectionMock con, DbDriverMock dbDriver, Table table)
		throws SQLException
	{
		Column[] columns = new Column[table.getNumColumns()];
		Table[] tables = new Table[] { table };
		
		for (int i = 0; i < table.getNumColumns(); ++i) {
			columns[i] = table.getColumn(i);
		}
		List<Row> result = dbDriver.select(con, columns, tables, null);
		assertNotNull(result);
	}
}
